#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#############################################################################################
# This plugin checks the running RD sessions on Remote Desktop Session Hosts (RDSH)         #
# You can define some usernames of users that must be online (e.g. system users for         #
# running specific scripts or execute specific tasks ....). You can find the regarding      #
# rule under Setup=>Service monitoring rules=>RDSH active user. In this rule you can define #
# usernames of one or more users! If one the users defined in this rule is offline the      #
# status of the check will switch to critical. If there is no rule defined then this check  #
# status will allways be OK!                                                                #
# ----------------------------------------------------------------------------------------- #
# Author: Michael Dvorak - specialis IT GmbH                                                #
# Version: 1.0 - 2021-12-23 - initial version                                               #
# Version: 1.1 - 2022-04-14 - Bugfix - copy user_patterns in variable                       #
#############################################################################################

# Example of section data:
# <<<spit_rdsh_user>>>
# steiner_robert Active 202112211120 STEINER-WS102 10.103.6.102
# Administrator Active 202112211331 DESKTOP-LGNR9K1 192.168.210.82
# ribar_maria Disconnected 202112211740

from .agent_based_api.v1 import *
import pprint

def discover_spit_rdsh_user(section):
    yield Service()

def check_spit_rdsh_user(params, section):

    if "user_patterns" in params:
        usernames = params["user_patterns"].copy()
        user_must_be_online = ",".join(params["user_patterns"])
    else:
        usernames = []
        user_must_be_online = "None"

    active=0
    disconnected=0
    actusers = []
    disusers = []
    act_user_details = []
    for username, sessionstate, logontime, workstation, ip in section:
        if sessionstate == "Active":
            active += 1
            logonTimeFormated = logontime[0:4] + '-' + logontime[4:6] + '-' + logontime[6:8] + ' ' + logontime[8:10] + ':' + logontime[10:12]
            act_user_details.append(f"{username} - Online since: {logonTimeFormated} - Workstation: {workstation} - IP: {ip}")
            actusers.append(username)
        elif sessionstate == "Disconnected":
            disconnected += 1
            disusers.append(username)
        # check if user is in array of usernames that must be online
        # if so, remove the username from the array
        if username in usernames:
            usernames.remove(username)
    yield Metric("active_users",active)
    yield Metric("disconnected_users",disconnected)

    disusernames = ','.join(disusers)
    actuserdetails = '\n'.join(act_user_details)
    # check if there are still usernames in the array of usernames that must be online 
    if len(usernames)>0:
        misusernames = ','.join(usernames)
        s = State.CRIT
        offline=len(usernames)
        mess_sum = f"Following user(s) must be online: {offline} - {misusernames}"
        mess_det = f"Active users: {active}\n{actuserdetails}\nDisconnected users: {disconnected} - {disusernames}\nMissing online user: {offline} - {misusernames}"
    else:
        s = State.OK
        mess_sum = f"Active users: {active} - Disconnected users: {disconnected} - Users which must be online: {user_must_be_online}"
        mess_det = f"Active users: {active}\n{actuserdetails}\nDisconnected users: {disconnected} - {disusernames}\nUsers which must be online: {user_must_be_online}"
    yield Result(state=s,summary=mess_sum,details=mess_det)
    
register.check_plugin(
    name="spit_rdsh_user",
    service_name="RDSH User",
    discovery_function=discover_spit_rdsh_user,
    check_function=check_spit_rdsh_user,
    check_default_parameters={},
    check_ruleset_name="spit_rdsh_user_params",
)
